'''
Created on Feb 15, 2019

@author: Bartosz Lew <bartosz.lew@umk.pl>
'''
import pkg_resources

import PyQt5.QtWidgets
from PyQt5 import QtWidgets 
from PyQt5 import QtCore, QtGui,uic
from PyQt5.QtCore import pyqtSignal,pyqtSlot, QSettings
import os,sys
import numpy as np
import scipy.io.wavfile as wf

from acomod import global_settings
from acomod import resources
from acomod import recordSignal
from acomod import preferences
from acomod import matplotlibWidget

class MainWindow(QtWidgets.QMainWindow):
    newSoundFileSelected = QtCore.pyqtSignal(str)
    
#     newSampleSignal = QtCore.pyqtSignal(list,list)

    def __init__(self,parent=None):
        super(MainWindow, self).__init__()
#         try:
#             self.ui=uic.loadUi('MainWindow.ui', self)
#         except:
        self.ui=uic.loadUi(pkg_resources.resource_filename('acomod','MainWindow.ui'), self)
            
#         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
#         self.ui=Ui_MainWindow()
#         self.ui.setupUi(self)
#         self.ui.pushButton.clicked.disconnect()
#         self.ui.pushButton.clicked.connect(self.on_pushButton_clicked)

        self.settings=global_settings.mySettings(self)
        prefs=preferences.PreferencesWindow()
        prefs.on_accept()
        # prefs.settings.sync()


        '''
        
        SETTING WIDGETS
        
        '''


        self.matplotlibWidget=matplotlibWidget.MatplotlibWidget(Npeaks=self.settings.value('Npeaks',1,type=int),
                                                                parent=self)
        self.ui.gridLayout.addWidget(self.matplotlibWidget,0,0)
#         self.ui.gridLayout.addWidget(QPushButton("1"),0,0)

#         self.matplotlibWidget.canvas.draw()
#         print(self.ui.doubleSpinBox_sampling.value())

        
        '''
        SETTINGS GUI STUFF
        '''
        
        

        self.ui.actionLogarithmic_scale_X.setChecked(self.settings.value('Logarithmic_scale_X',False,type=bool))
        self.ui.actionLogarithmic_scale_Y.setChecked(self.settings.value('Logarithmic_scale_Y',False,type=bool))
        self.ui.actionPlotAverage.setChecked(self.settings.value('PlotAverage',False,type=bool))
        self.ui.actionPlotMaximalValues.setChecked(self.settings.value('PlotMaximalValues',False,type=bool))
        self.ui.spinBox_Npeaks.setValue(self.settings.value('Npeaks',1,type=int))
        self.ui.doubleSpinBox_recordingLength.setValue(self.settings.value('recordingLength',1,type=float))

        self.sampling_freq=self.settings.value("sampling",44100, type=float)
        self.soundSpeed=self.settings.value("soundSpeed",343.0,type=float)
        self.recordingLength=self.settings.value("recordingLength",1.0,type=float)

        self.resize(self.settings.value("size", QtCore.QSize(840, 700)))
        self.move(self.settings.value("pos", QtCore.QPoint(50, 50)))


        '''
        
        CONNECTIONS
        
        '''
#         self.ui.
#         self.ui.actionLogarithmic_scale_X.toggled[bool].connect(self.on_actionLogarithmic_scale_X_toggled)
#         self.ui.actionLogarithmic_scale_Y.toggled[bool].connect(self.on_actionLogarithmic_scale_Y_toggled)
#         self.ui.actionPlotAverage.toggled[bool].connect(self.on_actionPlotAverage_toggled)
#         self.ui.actionPlotMaximalValues.toggled[bool].connect(self.on_actionPlotMaximalValues_toggled)
        self.ui.spinBox_Npeaks.valueChanged[int].connect(self.on_NpeaksValueChanged)

#         self.ui.actionPlay.toggled[bool].connect(self.on_actionPlay_toggled)

        '''
        
        STATUS BAR INFO
        
        '''
        self.statusBar = PyQt5.QtWidgets.QStatusBar()
        self.setStatusBar(self.statusBar)
#         self.statusBar.showMessage("Ready",2000)
        self.statusBar.showMessage("Ready")

        
#         self.ui.frame.setStyleSheet("background-color: rgb(200, 255, 255)")

#         plotWidget=MatplotlibWidget()
#         self.ui.frame.layoutVertical.addWidget(self.)
#         self.layoutVertical = QtGui.QVBoxLayout(self)



        '''
        
        Sources of sound
        
        '''
        self.soundFileName=None

#         self.show()
        
        
        '''
        
        Internal data
        
        '''
        self.recorded_data={'samples' : []}
        
    '''
    
    
    Slots and methods
    
    
    '''
        

    @QtCore.pyqtSlot()
    def show_statusBarMsg(self,msg,timeout=2000):
        print(msg)
        self.statusBar.showMessage(msg,timeout)


            
    def on_NpeaksValueChanged(self,n):
        settings=global_settings.mySettings(self)
        settings.setValue("Npeaks",n)
#         self.matplotlibWidget.selectMaxima(n)
#         self.matplotlibWidget.plotMaxima()
#         self.matplotlibWidget.draw()
        self.matplotlibWidget.mkplot()


    def cleanUp(self):
        if self.ui.actionRecord.isChecked():
            print("Disconnecting thread")
            self.dataTh.newSamplePowerSignal.disconnect()
            msg="Waiting for data thread to finish...please wait"
            print(msg)
            self.show_statusBarMsg(msg=msg,timeout=0)
#             self.dataTh.exit()
            event=QtCore.QEventLoop(self)
            self.dataTh.finished.connect(event.quit)
            self.dataTh.generateNew=False # let the thread die
            event.exec_()
            self.ui.actionRecord.toggled.disconnect()
            self.ui.actionRecord.setChecked(False)

    @QtCore.pyqtSlot()
    def dataTh_finished(self):
        print('data thread has finished')



#     @QtCore.pyqtSlot()
#     def Quit(self):
#         print("I am in quit")
#         self.close()
#         if self.dataTh.wait(1000):
#             self.dataTh.terminate()

    '''
    
    
    Events
    
    
    '''
    def closeEvent(self, ce):
        print("Close requested")
        self.cleanUp()
#         self.fileQuit()
#         self.ui.action_Quit.triggered.disconnect()
        print("Saving settings")
        self.settings.setValue("size", self.size())
        self.settings.setValue("pos", self.pos())
        self.settings.setValue("Npeaks",self.ui.spinBox_Npeaks.value())
        self.settings.setValue("recordingLength",self.ui.doubleSpinBox_recordingLength.value())

        del self.settings
#         self.on_action_Quit_triggered()
#         ce.ignore()
#         pass
        ce.accept()


    def keyPressEvent(self, event):
#         if event.key() == QtCore.Qt.Key_Q:
#             print "Killing"
#             self.deleteLater()
        if event.key() == QtCore.Qt.Key_Left:
            self.matplotlibWidget.previousMaximum()
            print('Current maximum:', self.matplotlibWidget.currentMaximum)
            self.matplotlibWidget.mkplot()
        elif event.key() == QtCore.Qt.Key_Right:
            self.matplotlibWidget.nextMaximum()
            print('Current maximum:', self.matplotlibWidget.currentMaximum)
            self.matplotlibWidget.mkplot()
        event.accept()


    '''
    
    
    Actions
    
    
    '''
        
    @QtCore.pyqtSlot(bool)
    def on_actionRecord_toggled(self, isOn):
#         self.matplotlibWidget.setFocus()
        self.ui.frame.setFocus()
        print(isOn)
        if isOn:
#             self.sampling_freq=self.settings.value("sampling",44100, type=float)
#             self.soundSpeed=self.settings.value("soundSpeed",343.0,type=float)
#             self.plotRefreshRate=self.settings.value("plotRefreshRate",1.0,type=float)
# #             self.ui.doubleSpinBox_length.value()
#             self.dataTh=recordSignal.newSample(update_time=self.plotRefreshRate,
#                                                sampling_freq=self.sampling_freq,
#                                                soundSpeed=self.soundSpeed,
# #                                                parent=self
#                                                )
#             self.dataTh=recordSignal.newSample(
# #                                                parent=self
#                                                )
            self.dataTh=recordSignal.newSoundSample(recordingLength=self.ui.doubleSpinBox_recordingLength.value())
            self.dataTh.newSamplePowerSignal.connect(self.matplotlibWidget.mkplot)
            self.dataTh.newSampleSignal.connect(self.on_new_sample_captured)
            self.dataTh.finished.connect(self.dataTh_finished)
            self.recorded_data={'samples' : []}

            self.matplotlibWidget.clearData()
            print("Starting data thread")
            self.dataTh.start()
            print("Thread started")
            self.statusBar.showMessage("Recording")
#             x=np.arange(1000)
#             y=np.sin(x)
#             self.newSampleSignal.emit(list(x),list(y))
#         self.matplotlibWidget.mkplot()
        else:
            self.show_statusBarMsg("Stopping data thread",0)
            event=QtCore.QEventLoop()
            self.ui.setEnabled(False)
            self.dataTh.finished.connect(event.quit)
            self.dataTh.generateNew=False # let the thread die
            event.exec_()
            self.ui.setEnabled(True)
            self.show_statusBarMsg("Ready")
#             self.dataTh.terminate()
#             self.dataTh.newSampleSignal.disconnect()
            self.ui.actionSaveAs.setEnabled(True)
        print("exiting slot")
        self.show_statusBarMsg("Use arrow keys to select maximum",0)
        
        
    @QtCore.pyqtSlot(bool)
    def on_actionPlay_toggled(self, isOn):
        self.ui.frame.setFocus()

        if isOn:
            self.dataTh=recordSignal.newSoundSample(soundFile=self.soundFileName,
                                                    recordingLength=self.ui.doubleSpinBox_recordingLength.value())
            self.dataTh.newSamplePowerSignal.connect(self.matplotlibWidget.mkplot)
            self.dataTh.finished.connect(self.dataTh_finished)
            self.matplotlibWidget.clearData()
            print("Starting data thread")
            self.dataTh.start()
            print("Thread started")
            self.show_statusBarMsg("Playing "+self.soundFileName,0)
#             x=np.arange(1000)
#             y=np.sin(x)
#             self.newSampleSignal.emit(list(x),list(y))
#         self.matplotlibWidget.mkplot()
        else:
            self.show_statusBarMsg("Stopping data thread",0)
            event=QtCore.QEventLoop()
            self.ui.setEnabled(False)
            self.dataTh.finished.connect(event.quit)
            self.dataTh.generateNew=False # let the thread die
            event.exec_()
            self.ui.setEnabled(True)
            self.show_statusBarMsg("Ready")
#             self.dataTh.terminate()
#             self.dataTh.newSampleSignal.disconnect()
        print("exiting slot")
        
        
    @QtCore.pyqtSlot()
    def on_actionOpenFile_triggered(self):
#         fname = QtWidgets.QFileDialog.getOpenFileName(self, 'Open file', '.',"Audio files (*.wav)")        
#         print(fname)

        options = QtWidgets.QFileDialog.Options()
        options |= QtWidgets.QFileDialog.DontUseNativeDialog
        mysettings=global_settings.mySettings()
        prevFileName=mysettings.value("LastOpenedFile",".",type=str)
        print('prevFileName: %s' % prevFileName)
#         prevFileName=""
        fileName, _ = QtWidgets.QFileDialog.getOpenFileName(self,"Select audio file to load", prevFileName,"Audio files (*.wav);;All Files (*)", options=options)
        if fileName:
            print(fileName)
            mysettings.setValue("LastOpenedFile",fileName)
            self.soundFileName=fileName
            self.show_statusBarMsg("Selected sound file: "+fileName)
            self.newSoundFileSelected.emit(fileName)
            self.ui.actionPlay.setEnabled(True)
            self.ui.actionSaveAs.setEnabled(True)

    @QtCore.pyqtSlot()
    def on_actionSaveAs_triggered(self):
#         fname = QtWidgets.QFileDialog.getOpenFileName(self, 'Open file', '.',"Audio files (*.wav)")        
#         print(fname)

        options = QtWidgets.QFileDialog.Options()
        options |= QtWidgets.QFileDialog.DontUseNativeDialog
        mysettings=global_settings.mySettings()
        prevFileName=mysettings.value("LastSavedFile",".",type=str)
        print('prevFileName: %s' % prevFileName)
#         prevFileName=""
#         saveFileName = QtWidgets.QFileDialog.getExistingDirectory(self,"Select destination folder", prevFileName, options=options)
        saveFileName = QtWidgets.QFileDialog.getSaveFileName(self, "Save File",
                           prevFileName,options=options);
        if saveFileName[0]!='':
            saveFileName=saveFileName[0]
            print(saveFileName)
            mysettings.setValue("LastSavedFile",saveFileName)
            self.soundFileName=saveFileName
#             self.show_statusBarMsg("Not implemented yet")
#             self.show_statusBarMsg("Selected sound file: "+saveFileName)
#             self.newSoundFileSelected.emit(saveFileName)
#             self.ui.actionPlay.setEnabled(True)
            allsamples=[]
            for s in self.recorded_data['samples']:
                allsamples+=s
            print("All samples length {}".format(len(allsamples)))
            allsamples=np.asanyarray(allsamples, dtype=float)
            wf.write(saveFileName,int(mysettings.value('sampling',type=float)),allsamples)
            self.show_statusBarMsg("Saving to file {}".format(saveFileName))

        
    @QtCore.pyqtSlot()
    def on_action_Quit_triggered(self):
        self.cleanUp()
        
        self.close()


    @QtCore.pyqtSlot()
    def on_actionPreferences_triggered(self):
        prefs=preferences.PreferencesWindow()
#         prefs.preferencesSignal.connect(self.show_statusBarMsg)
        if prefs.exec_():
            self.show_statusBarMsg("Preferences saved")
        else:
            self.show_statusBarMsg("Cancelled")

#         prefs.show()



    @QtCore.pyqtSlot()
    def on_action_About_triggered(self):
        QtWidgets.QMessageBox.about(self, "About", "Resonant Frequency Viewer (v."+global_settings.PROGRAM_VERSION+")"+"""        

This program calculates and visualizes spectrum of acoustic waves provided from a microphone or from a sound file. 
It shows acoustic modes of resonating or rotating objects and relation to their length scales (dimensions) or rotation frequencies via provided speed of sound.

For example, the program can be used to estimate length of an excited metal bar, and to identify resonance frequencies
and the corresponding length scales of mechanical components (e.g. in a running car) or to test 1/f noise and microphonic effects in electrical devices the program runs on.

Copyright (2019) Bartosz Lew (bartosz.lew@protonmail.com)

"""
                                )



    @QtCore.pyqtSlot(bool)
    def on_actionLogarithmic_scale_X_toggled(self,tf):
        settings=global_settings.mySettings(self)
        settings.setValue('Logarithmic_scale_X',tf)
        self.matplotlibWidget.configurePlotAxes()

    @QtCore.pyqtSlot(bool)
    def on_actionLogarithmic_scale_Y_toggled(self,tf):
        settings=global_settings.mySettings(self)
        settings.setValue('Logarithmic_scale_Y',tf)
        self.matplotlibWidget.configurePlotAxes()

    @QtCore.pyqtSlot(bool)
    def on_actionPlotAverage_toggled(self,tf):
        settings=global_settings.mySettings(self)
        settings.setValue('PlotAverage',tf)
        self.matplotlibWidget.mkplot()

    @QtCore.pyqtSlot(bool)
    def on_actionPlotMaximalValues_toggled(self,tf):
#         print(tf)
        settings=global_settings.mySettings(self)
        settings.setValue('PlotMaximalValues',tf)
        self.matplotlibWidget.mkplot()

    @QtCore.pyqtSlot(bool)
    def on_actionFocus_on_plot_triggered(self,tf):
        self.ui.frame.setFocus()
        self.show_statusBarMsg("Use arrow keys to select maximum",0)

        
    @QtCore.pyqtSlot(list,list)
    def on_new_sample_captured(self,t,sig):
#         self.recorded_data['sample']=np.vstack([np.array(t),np.array(sig)]).T
        self.recorded_data['samples'].append(sig)

        print("Captured new sound sample ({} samples, duration {} s".format(
            len(t),t[-1]))
        print(len(self.recorded_data['samples']))

#     def write_to_file(self,):



'''
Created on Feb 12, 2019

@author: Bartosz Lew <bartosz.lew@umk.pl>
'''

import os,sys
from PyQt5 import QtCore
PROGRAM_NAME="Acoustic Oscillations Viewer"
COMPANY_NAME="bslew"
PROGRAM_VERSION='0.1 alpha'

class mySettings(QtCore.QSettings):
    def __init__(self,parent=None):
        super(mySettings,self).__init__(COMPANY_NAME,PROGRAM_NAME, parent=parent)





'''
Created on Feb 11, 2019

@author: blew
'''
import pkg_resources
from PyQt5 import QtWidgets
from PyQt5 import QtCore, QtGui,uic
from PyQt5.QtCore import QSettings
from acomod import global_settings

class PreferencesWindow(QtWidgets.QDialog):
    preferencesSignal = QtCore.pyqtSignal(str)

    def __init__(self,parent=None):
        super(PreferencesWindow, self).__init__()
        try:
            self.ui=uic.loadUi('preferences.ui', self)
        except:
            self.ui=uic.loadUi(pkg_resources.resource_filename('acomod','preferences.ui'), self)
#         self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
        settings=global_settings.mySettings(self)
        self.ui.doubleSpinBox_sampling.setValue(settings.value('sampling',44100,type=float))
        self.ui.doubleSpinBox_plotRefreshRate.setValue(settings.value('plotRefreshRate',5,type=float))
        self.ui.doubleSpinBox_soundSpeed.setValue(settings.value('soundSpeed',343,type=float))
        self.ui.spinBox_plotPointsCount.setValue(settings.value('plotPointsCount',500,type=int))
        self.ui.spinBox_maximumMinPoints.setValue(settings.value('maximumMinPoints',10,type=int))
        self.ui.buttonBox.accepted.connect(self.on_accept)
        self.ui.buttonBox.rejected.connect(self.on_reject)


    @QtCore.pyqtSlot()
    def on_accept(self):
        settings=global_settings.mySettings(self)
        settings.setValue('sampling',self.ui.doubleSpinBox_sampling.value())
        settings.setValue('plotRefreshRate',self.ui.doubleSpinBox_plotRefreshRate.value())
        settings.setValue('soundSpeed',self.ui.doubleSpinBox_soundSpeed.value())
        settings.setValue('plotPointsCount',self.ui.spinBox_plotPointsCount.value())
        settings.setValue('maximumMinPoints',self.ui.spinBox_maximumMinPoints.value())

#         self.settings.writeSettings()
#         self.settings.sync()
#         del self.settings
        print("Settings saved")
        self.preferencesSignal.emit("Settings saved") # why connecting to this signal from the
        # invoking class causes segfault ?
        
    @QtCore.pyqtSlot()
    def on_reject(self):
        print("Settings not saved")
        self.preferencesSignal.emit("Settings not saved")



'''
Created on Feb 11, 2019

@author: blew
'''
from PyQt5 import QtWidgets, QtCore, QtGui,uic
import sys

from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
import numpy as np
import sounddevice as sd
from acomod import global_settings
import copy
from scipy.signal import argrelextrema

class MatplotlibWidget(FigureCanvas):
    def __init__(self, parent=None, width=5, height=4, dpi=100, Npeaks=1):
        super(MatplotlibWidget, self).__init__(Figure(figsize=(width, height), dpi=dpi))
        self.setParent(parent)
        
        fig = self.figure
        self.axes = fig.add_subplot(111)
        self.settings=global_settings.mySettings(self)
        self.cs=self.settings.value("soundSpeed",type=float)
        self.Npeaks=Npeaks
        
#         self.

#         self.mkplot()

#         FigureCanvas.__init__(self, fig)

        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding,QtWidgets.QSizePolicy.Expanding)
        self.updateGeometry()

        self.updatablePlotArtists=[]
        self.plotLabels=None
        self.rawFnPlot=None
        self.avgFnPlot=None
        self.maxFnPlot=None
        self.rawFn=None
        self.avgFn=None
        self.sumFn=None
        self.maxFn=None
        self.dataCount=0
#         self.axis.clear()

        self.currentMaximum=0
        self.configurePlotAxes()



    def configurePlotAxes(self):
        settings=global_settings.mySettings(self)
        self.plotPointsCount=settings.value("plotPointsCount",type=int)
        self.axes.set_xlabel('Frequency [Hz]')
        self.axes.set_ylabel('Power [$Hz^{-1}$]')
        if settings.value("Logarithmic_scale_X",type=bool):
            self.axes.set_xscale("log")
        else:
            self.axes.set_xscale("linear")
            
        if settings.value("Logarithmic_scale_Y",type=bool):
            self.axes.set_yscale("log")
        else:
            self.axes.set_yscale("linear")
        
        self.resetRanges()
        self.draw()
         
    @QtCore.pyqtSlot()
    def clearData(self):
        self.rawFn=None
        self.sumFn=None
        self.maxFn=None
        self.plotLabels=None
        self.dataCount=0
        self.selectedMaxima=None

    def find_maxima(self, fn,Npeaks,maximumMinPoints=10):
        m,=argrelextrema(fn[:,1], np.greater,order=maximumMinPoints)
#         print(m)
#         print(len(m))
        
        return fn[:,0][m],fn[:,1][m]
#         pass

    def plot_labels(self,X,Y):
        if not hasattr(X,'__iter__'):
            X=[X]
        if not hasattr(Y,'__iter__'):
            Y=[Y]
#         print(X,Y)
        for x,y in zip(X,Y):
            vl=self.axes.axvline(x,lw=1, color='c')
            self.updatablePlotArtists.append(vl)
            l=self.cs/x
#             offset=10
            t=self.axes.text(x,y,' {0:.1f} Hz\n {1:.3f} m'.format(x,l), fontsize=14)
#             t=self.axes.text(x,y,'{0:.1f} Hz'.format(x))
            self.updatablePlotArtists.append(t)
            


    def binFn(self,Fn):
#         bs=len(Fn) // self.plotPointsCount
        settings=global_settings.mySettings(self)
        self.plotPointsCount=settings.value("plotPointsCount",type=int)
        
        f,e1=np.histogram(Fn[:,0], self.plotPointsCount,weights=Fn[:,1])
        x=[(e1[i]+e1[i+1])/2 for i in range(len(e1)-1)]
#         y=[(y[i]*e2[i]/+y[i+1])/2 for i in range(len(Fn-1))]
        return x,f

    def resetRanges(self):
        if type(self.rawFn)!=type(None):
            self.axes.set_xlim(min(self.rawFn[:,0]),max(self.rawFn[:,0]))        

#             if type(self.maxFn)!=type(None):
            self.axes.set_ylim(min(self.rawFn[:,1]),max(self.maxFn[:,1]))        


    def selectMaxima(self,n):
        if n>0:
            self.selectedMaxima=np.sort(self.sortedMaxima[:self.Npeaks],order='f')
        else:
            self.selectedMaxima=[]
            
    def plotMaxima(self):
        plotLabels, =self.axes.plot(self.selectedMaxima['f'],self.selectedMaxima['P'],'ko')
        self.updatablePlotArtists.append(plotLabels)
        
    def nextMaximum(self):
        self.currentMaximum=(self.currentMaximum+1) % len(self.selectedMaxima)

    def previousMaximum(self):
        self.currentMaximum=(self.currentMaximum-1) % len(self.selectedMaxima)

    @QtCore.pyqtSlot(list,list)
    def mkplot(self,x=None,y=None):
        if type(x)!=type(None) and type(y)!=type(None):
            x=np.asarray(x,dtype=float)
            y=np.asarray(y,dtype=float)
            self.rawFn=np.vstack((x,y)).T
    #         sys.exit()
            # calculate sum
            if type(self.sumFn)==type(None):
                self.sumFn=np.array([x,y],dtype=float).T
                self.maxFn=np.zeros(len(self.sumFn)*2).reshape(-1,2)
            else:
                self.sumFn+=np.array([x,y],dtype=float).T
            self.dataCount+=1
                
                
            # calculate average
            self.avgFn=self.sumFn/self.dataCount
    
            # calculate maximal values
            self.maxFn=np.array([ [x[0],max(x[1],y[1])] for x,y in zip(self.rawFn,self.maxFn) ])
    #         print(self.maxFn)



        #
        # find maxima
        # 
        self.Npeaks=self.settings.value("Npeaks",10,int)
        if type(self.maxFn)!=type(None):
            maximumMinPoints=self.settings.value("maximumMinPoints",10,type=int)
            maxx,maxy=self.find_maxima(self.maxFn,self.Npeaks,maximumMinPoints)

            L=[ self.cs/x for x in maxx]
    #             t=self.axes.text(x,y,'{0:.1f} Hz\n{1:.3f} m'.format(x,l))
            self.sortedMaxima=np.sort(np.array([(x,y,l) for x,y,l in zip(maxx,maxy,L)], dtype=[('f',float),('P',float),('L',float)]), 
                                 axis=0, order='P')[::-1]
            self.selectMaxima(self.Npeaks)
    #         self.sortedMaxima=np.sort(self.sortedMaxima, order='f')
    #         print(sortedMaxima)
    #         print(sortedMaxima['x'],sortedMaxima['y'])
        
            if self.Npeaks>0:
                print("Maxima frequencies and corresponding lengths, sorted according to decreasing spectral power [(Hz,m)]:")
                print(self.selectedMaxima[['f','L','P']])





        


        #
        # remove updatable artists            
        #
        while len(self.updatablePlotArtists)>0:
            self.updatablePlotArtists[0].remove()
            self.updatablePlotArtists.pop(0)

#         if self.rawFnPlot!=None:
#             self.rawFnPlot.remove()
#             self.rawFnPlot=None # this is not needed but for purity

        #
        # plot
        #
        if type(self.maxFn)!=type(None):
            self.plotPointsCount=self.settings.value("plotPointsCount",500,type=int)
            bs=len(self.rawFn)//self.plotPointsCount
            if bs<1:
                bs=1
    #         x,y=self.binFn(np.array([x,y],dtype=float).T)
            x,y=self.rawFn[::bs][:,0],self.rawFn[::bs][:,1]
            rawFnPlot, =self.axes.plot(x,y,'k-')
            self.updatablePlotArtists.append(rawFnPlot)
    
            if self.settings.value("PlotAverage",type=bool):
                x,y=self.avgFn[::bs][:,0],self.avgFn[::bs][:,1]
    #             x,y=self.binFn(self.avgFn)
                avgFnPlot, =self.axes.plot(x,y,'g-', zorder=10, lw=2)
    #             avgFnPlot, =self.axes.plot(self.avgFn[:,0],self.avgFn[:,1],'g-', zorder=10, lw=2)
                self.updatablePlotArtists.append(avgFnPlot)
            if self.settings.value("PlotMaximalValues",type=bool):
                x,y=self.maxFn[::bs][:,0],self.maxFn[::bs][:,1]
    #             x,y=self.binFn(self.maxFn)
                maxFnPlot, =self.axes.plot(x,y,'r-', zorder=20)
    #             avgFnPlot, =self.axes.plot(self.maxFn[:,0],self.maxFn[:,1],'r-', zorder=20)
                self.updatablePlotArtists.append(maxFnPlot)
    



            #
            # plot maxima
            #
            if self.Npeaks>0:
                self.plotMaxima()

        
            #
            # plot labels
            #
    #         self.plot_labels(self.selectedMaxima['f'],self.selectedMaxima['P'])
    #         print(self.selectedMaxima['P'][self.currentMaximum])
            if self.Npeaks>0:
                if self.currentMaximum>=self.Npeaks:
                    self.currentMaximum=self.Npeaks-1
                if len(self.selectedMaxima['f'])>self.currentMaximum:
                    self.plot_labels(self.selectedMaxima['f'][self.currentMaximum],self.selectedMaxima['P'][self.currentMaximum])
    
            self.resetRanges()
            self.draw()
        
#         sd.play(y, 11600)
#         sd.default.samplerate = self.ui.doubleSpinBox_sampling.value()
#         sd.play(y)

        



'''
Created on Feb 10, 2019

@author: Bartosz Lew <bartosz.lew@protonmail.com>
'''
import os,sys
sys.path.append(os.sep.join(os.path.abspath(__file__).split(os.sep)[:-2]))  

# from PyQt5 import sip
from PyQt5 import QtWidgets 
from acomod import MainWindow

sys._excepthook = sys.excepthook
def exception_hook(exctype, value, traceback):
    sys._excepthook(exctype, value, traceback)
    sys.exit(1)
sys.excepthook = exception_hook



def main():
    app = QtWidgets.QApplication(sys.argv)
    progMainWindow = MainWindow.MainWindow()
    progMainWindow.show()
    try:
        sys.exit(app.exec_())
    except:
        print("exiting")
        raise

if __name__ == '__main__':
    main()
    


'''
Created on Feb 11, 2019

@author: blew
'''

from PyQt5 import QtCore
# from PyQt5.QtCore import QThread,QTimer

import sounddevice as sd
import numpy as np
import random
import time
from scipy.signal import periodogram
from acomod import global_settings
import sys
import copy

# class newSample(QtCore.QThread):
#     newSamplePowerSignal = QtCore.pyqtSignal(list,list)
#     
#     
# #     def __init__(self, update_time=0.5, sampling_freq=44100, soundSpeed=343, parent=None):
#     def __init__(self, update_time=0.5, sampling_freq=44100, soundSpeed=343, parent=None):
#         '''
#         update_time [s] - time after which new sample will be generated
#         '''
#         super(newSample,self).__init__(parent)
#         settings=global_settings.mySettings(self)
# 
#         self.update_time=1.0/settings.value('plotRefreshRate',type=float)
#         self.sampling_freq=settings.value('sampling',type=float)
#         self.soundSpeed=settings.value('soundSpeed',type=float)
#         self.plotPointsCount=settings.value('plotPointsCount',500,type=int)
#         self.generateNew=True
# #         self.timer=QtCore.QTimer(self)
# #         print('update time: {}'.format(int(update_time*1000)))
# #         self.timer.setInterval(int(update_time*1000))
# #         self.timer.timeout.connect(self.sendNewSample)
#         
#     
#     def run(self):
#         print("New thread")
# 
# #         self.timer.start()
#         while self.generateNew:
# #             print("data thread")
#             self.sendNewSample()
#             settings=global_settings.mySettings(self)
#             time.sleep(1./settings.value("plotRefreshRate",type=float))
# #             time.sleep(1000)
#         
#     
#     
#     def getFFT(self,y,sampling=1,maxPoints=1000):
# #         ft=np.fft.fft(y)
# #         n=len(y)
# #         L=1.0/(sampling*n)
# #         p=ft.real*ft.real+ft.imag*ft.imag
# #         print(len(y))
# #         print(len(p))
# #         f=[1.0/L for x in range(n)]
# #         print(p)
# 
#         p=periodogram(y, fs=sampling,nfft=maxPoints)
# #         print(p)
#         x=p[0][1:] # drop 0th frequency
#         y=p[1][1:]
#         return x,y
# 
# 
#     def getNewData(self,n):
#         x=np.arange(n, dtype=float)
#         noise=np.asarray([random.random() for _ in range(n)],dtype=float)
#         y=np.sin(x)+noise*0.1
#         return x,y
# 
# 
#     def getSoundData(self,n):
#         duration=self.update_time
#         fs=self.sampling_freq
#         myrecording = sd.rec(int(duration * fs), samplerate=fs, channels=2)        
#         myrecording=np.mean(myrecording,axis=1)
#         t=np.arange(0.0,duration,1.0/fs)
#         print(myrecording)
#         return t,myrecording
# 
#     
#     @QtCore.pyqtSlot()
#     def sendNewSample(self):
#         n=500
# #         x,y=def.getNewData(n)
#         x,y=self.getSoundData(n)
# 
#         
#         x,y=self.getFFT(y,self.sampling_freq,maxPoints=self.plotPointsCount)
#         
# #         print("Emitting new sample")
#         self.newSamplePowerSignal.emit(list(x),list(y))
#         
        


class newSoundSample(QtCore.QThread):
    newSamplePowerSignal = QtCore.pyqtSignal(list,list)
    newSampleSignal = QtCore.pyqtSignal(list,list)
    
    
    def __init__(self, update_time=0.5, sampling_freq=44100, soundSpeed=343, recordingLength=1.0, soundFile=None, parent=None):
        '''
        update_time [s] - time after which new sample will be generated
        '''
        if sys.version_info[0] < 3:
            super(newSoundSample,self).__init__(parent)
        else:
            super().__init__(parent)
        settings=global_settings.mySettings(self)

#         self.update_time=1.0/settings.value('plotRefreshRate',type=float)
        self.sampling_freq=settings.value('sampling',sampling_freq,type=float)
        self.soundSpeed=settings.value('soundSpeed',soundSpeed,type=float)
        self.plotPointsCount=settings.value('plotPointsCount',500,type=int)
        self.recordingLength=recordingLength
        
        self.generateNew=True
        self.soundFile=soundFile
        
        self.recordingTime=[]
        self.recordingData=[]
        self.recordingDataAll=[]
        

        
    def callback(self,indata, frames, time, status):
#         fftsize=500
#         gain=1.0
#         if status:
#             text = ' ' + str(status) + ' '
#             print('\x1b[34;40m', text.center(40, '#'),
#                   '\x1b[0m', sep='')
#         if any(indata):
#             magnitude = np.abs(np.fft.rfft(indata[:, 0], n=fftsize))
#             magnitude *= gain / fftsize
#             print(magnitude)
#         print(indata)
#         print(np.shape(indata))
        x,y=self.getFFT(indata[:,0],self.sampling_freq,maxPoints=self.plotPointsCount)
        self.recordingData=copy.copy(indata[:,0])
#         self.recordingDataAll.append(self.recordingData)
        
#         else:
#             print('no input')
  
        self.newSamplePowerSignal.emit(list(x),list(y))
#         print('emitting')
#         print(list(self.recordingTime))
#         print(list(self.recordingData))
        self.newSampleSignal.emit(list(self.recordingTime),list(self.recordingData))
        
        
    def runSoundFileSample(self):
        print("New sound file thread")
        
        
#         import sounddevice as sd
        import soundfile as sf
        
        data, fs = sf.read(self.soundFile, dtype='float32')
        x=list(np.arange(0.0,self.recordingLength,1.0/fs))
        print('file samples shape: ',np.shape(data))

        if len(np.shape(data))>1:
            if np.shape(data)[1]==2:
                print("file is stereo, converting to mono...")
                data=np.mean(data,axis=1)
#         data=data.reshape(len(data),1)
        print('file sampling rate: ',fs)
        N=len(data)
        print("file samples count: ", N)        
        i=0
        n=len(x)
        Nsubsamples=N//n
        print("sub-sample length: ",self.recordingLength)
        while self.generateNew:
            if i==Nsubsamples:
                i=0
            sample=np.roll(data,-n*i,axis=0)[0:n]
            sample=data[i*n:(i+1)*n]
            
#             print("sub-sample length: ",len(sample))
#             print(sample)

#             x,y=self.getFFT(sample,fs,maxPoints=self.plotPointsCount)
            x,y=self.getFFT(sample,fs)
            self.newSamplePowerSignal.emit(list(x),list(y))
            i+=1

#             settings=global_settings.mySettings(self)
#             time.sleep(self.update_time)
            time.sleep(self.recordingLength)
        
        
#         import sounddevice as sd
#         import soundfile as sf
#         import queue
#          
#         self.device=7
#         blockSize=int(self.sampling_freq* self.update_time)
#         bufferSize=blockSize*1000
#         print(bufferSize)
#         print(blockSize)
#         q = queue.Queue(maxsize=bufferSize)
#         with sf.SoundFile(self.soundFile) as f:
#             for _ in range(bufferSize):
#                 data = f.buffer_read(int(blockSize), dtype='float32')
# #                 data = f.buffer_read(1, dtype='float32')
#                 if not data:
#                     break
#                 q.put_nowait(data)  # Pre-fill queue
# 
#      
#             stream = sd.RawOutputStream(
#                 samplerate=f.samplerate, blocksize=blockSize,
#                 device=self.device, channels=f.channels, dtype='float32',
#                 callback=self.callback, 
# #                 finished_callback=event.set
#                 )
#  
#             with stream:
#                 timeout = blockSize * bufferSize / f.samplerate
#                 while data:
#                     data = f.buffer_read(blockSize, dtype='float32')
#                     q.put(data, timeout=timeout)
# #                 event.wait()  # Wait until playback is finished    
#              
#                     if self.generateNew==False:
#                         break
    
    def runInputStreamSample(self):
        print("New input stream thread")
        self.device=7

        buff_size=self.sampling_freq*self.recordingLength
        self.recordingTime=np.arange(0,self.recordingLength,1./self.sampling_freq)
#         print(self.recordingTime)

        with sd.InputStream(
                            channels=1, 
#                             device=self.device, 
                            callback=self.callback,
                            blocksize=int(buff_size),
                            samplerate=self.sampling_freq):
            print(self.sampling_freq*self.recordingLength)
            while self.generateNew:
#                 print("generate new")
#                 settings=global_settings.mySettings(self)
#                 time.sleep(1./settings.value("plotRefreshRate",type=float))
#                 time.sleep(self.update_time)
                time.sleep(0.1)
        
    
    def run(self):
        
        if self.soundFile!=None:
            self.runSoundFileSample()
        else:
            self.runInputStreamSample()
    
    
    def getFFT(self,y,sampling=1,maxPoints=1000):
#         ft=np.fft.fft(y)
#         n=len(y)
#         L=1.0/(sampling*n)
#         p=ft.real*ft.real+ft.imag*ft.imag
#         print(len(p))
#         f=[1.0/L for x in range(n)]
#         print(p)

#         p=periodogram(y, fs=sampling,nfft=maxPoints)
        p=periodogram(y, fs=sampling)
#         print(p)
        x=p[0][1:] # drop 0th frequency
        Y=p[1][1:]
#         print(x)
        print("FFT: %i, min./max.freq.: %f/%f" % (len(y),min(x),max(x)))
        return x,Y


#     def getNewData(self,n):
#         x=np.arange(n, dtype=float)
#         noise=np.asarray([random.random() for _ in range(n)],dtype=float)
#         y=np.sin(x)+noise*0.1
#         return x,y


#     def getSoundData(self,n):
#         duration=self.update_time
#         fs=self.sampling_freq
#         myrecording = sd.rec(int(duration * fs), samplerate=fs, channels=2)        
#         myrecording=np.mean(myrecording,axis=1)
#         t=np.arange(0.0,duration,1.0/fs)
#         print(myrecording)
#         return t,myrecording
# 
#     
#     @QtCore.pyqtSlot()
#     def sendNewSample(self):
#         n=500
# #         x,y=def.getNewData(n)
#         x,y=self.getSoundData(n)
# 
#         
#         x,y=self.getFFT(y,self.sampling_freq,maxPoints=self.plotPointsCount)
#         
# #         print("Emitting new sample")
#         self.newSamplePowerSignal.emit(list(x),list(y))
        
        

        
        


from setuptools import setup
import os

import codecs
try:
    codecs.lookup('mbcs')
except LookupError:
    ascii = codecs.lookup('ascii')
    func = lambda name, enc=ascii: {True: enc}.get(name=='mbcs')
    codecs.register(func)
    
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

required_packages=read('requirements.txt').split()

setup(name='acomod',
      version='0.1.12',
      description='Acoustic Oscillations Viewer',
      long_description=read('README.md'),
      long_description_content_type='text/markdown',
      author='Bartosz Lew',
      author_email='bartosz.lew@protonmail.com',
      url='https://github.com/bslew/acomod',
      install_requires=required_packages,
      package_dir = {'': 'src'},
      packages = ['acomod',
                  ],
      entry_points={ 'gui_scripts': [ 'acoustic_mode_viewer = acomod.sonidist:main',
               ],
                    },
      package_data={'acomod': ['*.ui','resources.qrc'
                               ]},
#       eager_resources={'acomod': ['data/*.wav','data/*.WAV',
#                                ]},
    data_files=[('data', ['src/acomod/data/cow.wav',
                             ])],
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License"
        ],
     )
